'************************************************************************************
' Expand.vba 
'
' This utility demonstrates how to establish a connection with a host through port 25 and 
' get the members of a mail group using the 'expn' command.
' The script is composed of 4 parts:
' 
' 1. 	Establish a new connection:
' The user gave a name of a new host. A new conenction is made and the alias group given is expanded
'
' 2.	Expand a new alias group :
' The user gave a different name of group but didn't change the host name. The current connection
' is kept and a new alias group is expanded.
'
' 3.	Send mail:
' Sending mail to the highlighted address in the list.
'								      
' 4. 	Main:
'
' The main part invoked the dialog that asks for the host and alias names. It calls the other subroutines
' according to the user's selection.
'
' Copyright (C) 1996 NetManage Inc.
' All rights reserved.
'
' This code may be renamed and modified for your own purposes.
' ************************************************************************************

' The Constants are used to identify the host state during script execution. You should change 
' them if the host you are using produces other codes.

Global Const Connection = "220"	     
Global Const MessagePrompt="250"    
Global Const ErrorMessage="500"

' Most of the variables are global becuse both expand subroutines needs them and it is easier to 
' manage the table of addresses this way. However, generally it is better not to use global 
' variables for more complex scripts.

Dim length As Integer		' Holds the length of the current address
Dim search (1) As String  	' The searched number 
Dim table(300) as string		' Addresses Table
Dim host As string			' Host name
Dim aliasName As string	' Alias name
Dim temp As String	
Dim rightTr As String
Dim leftTr As String
Dim message As String	
Dim result As integer		' Holds the EmWait results

Sub  Main  ()
	SysHide  "Timer" 
	dim button as integer
	Emulation"Telnet"
	Emdisconnect "AZ"
	
' Dialog definition
ShowDialog:

Begin Dialog MAINMENU 66,42, 256, 200, "EXPAND"
  Text 130,24,70,10, "Host:"
  TextBox 130,38,112,12, .host
  Text 130,74,26,10, "Alias:"
  TextBox 130,88,110,12, .aliasName
  ListBox 12,24,110,140, table(), .Lstbox
  PushButton 12,178,40,14, "&Expand", .expand
  PushButton 82,178,40,14, "&Send mail", .sendmail
  CancelButton 200,178,40,14
End Dialog


Dim Dlg As MainMenu

' Activate the dialog and get the button that was clicked.
button = Dialog (Dlg)
aliasName=Dlg.aliasName

' Cancel was clicked 
if button = 0 then 
	EmDisconnect "AZ"
	kill "file.txt"
	Stop
End If

' Don't precede if nothing was written in the host place
if Dlg.host = "" Then goto ShowDialog

' Expand was clicked. If the current host name as the same as the last expand then goto ExpandNew
' If the host name was changes goto NewConnection
if button = 1 Then 
			If Dlg.host = host Then 
						ExpandNew
			Else
						NewConnection
			End If
End If

' Send Mail was clicked and a name was highlighted
If button = 2 Then 
		if table(Dlg.lstbox)<>"" then
						SendMail
		End If
End If 
Goto ShowDialog
End Sub

' ***********************************************************************************
' * Subroutine : NewConnection  
' * Purpose:  Establishing a new connection 
' *
' ***********************************************************************************

Sub NewConnection
	Dim i As integer	
	Dim l As integer
	EmDisconnect "AZ" 		
	SysDelay 1
	host=Dlg.host	

' prepare the table for the new list, set connection parameters
	Erase table 			
	Emulation  "TELNET"   
	EMSetTelnet  host,25  

' Wait for the initial connection
	search(1)=Connection		
	EmWaitLineTexts "Set",a,b,search	
	EMActivate  "AZ"

' Hide Emulation window,Send "helo" to host	
	SysHide "Emulation"
	Search(1) = MessagePrompt
	EmWaitLineTexts "Set", 1, 1, search
	EMSendKey "helo"
	EMSendKey  "<CTRL_J>"  				
	EmWaitLineTexts "Wait", 30, result, search	' Wait for the response

' Send the expn command along with the alias name
	search(1) = ErrorMessage
	EmWaitLineTexts "Set", 30, result, search
	message = "expn " & aliasName
	EmSendKey message
	EMSendKey   "<CTRL_J>" 
	
' There is no command such as "dummy". It is a dummy command sent to the host in order to 
' determine  when  it finished sending the names. When it is finished it tries to interpert the 
' dummy command and gives an error message. This error message that starts with "500" 
' indicates that all the addresses has been sent.
	
	EmSendKey "dummy"		
	EmSendKey "<CTRL_J>"
	
' Start capturing incoming text to file till the error code is sent
	EMCaptureOn  "file.txt", "Overwrite"
	EmWaitLineTexts "Wait", 30, result, search
	EmCaptureOff

' Manipulating the captured text into the table
	FileRead  "file.txt" , temp, 0

' Skip any spaces in the beginning of the file
	While (temp = "")
		FileReadNext temp, 0
	Wend

' Trunctate any leading or trailing spaces 
	leftTr = Ltrim$(temp)
	rightTr = Rtrim$(leftTr)

' The error message is captured also, so this is the indicator for the last record 
	While (Left$(rightTr, 3) <> ErrorMessage) 
		l = len(rightTr)		
		l = l - 4    			
		if l < 1 then goto OutOfArray	                                      
		table(i) = Right$(rightTr,l) 
		i = i + 1

' Check that the number of records is less the 300 ( the array size )
		if i > 300 Then Goto OutOfArray	
		FileReadNext temp,0
		While (temp = "")
			FileReadNext temp, 0
		Wend
		leftTr = Ltrim$(temp)
		rightTr = Rtrim$(leftTr)
	Wend

OutOfArray:
End Sub

' ***********************************************************************************
' * Subroutine : ExpandNew  
' * Purpose:  Expand a new alias name from the same host 
' *
' ***********************************************************************************
Sub ExpandNew
	Dim i,l As integer	

' prepare the table for the new list
	Erase table

' Send the expn alias name command. See the explanation in NewConnection for the use of 
' the "dummy" string
	search(1)=ErrorMessage
	EmWaitLineTexts "Set", 60, result, search
	message = "expn " & aliasName
	EmSendKey message
	EMSendKey "<CTRL_J>" 
	EmSendKey "dummy"
	EmSendKey "<CTRL_J>"
	
' Start capturing incoming text to file till the error code is sent
	EMCaptureOn "file.txt", "Overwrite"
	EmWaitLineTexts "Wait", 60, result, search
	EmCaptureOff

' Manipulating the captured text into the table	
	FileRead  "file.txt", temp, 0

' Skip any spaces in the beginning of the file
	While (temp = "")
		FileReadNext temp, 0
	Wend

' Trunctate any leading or trailing spaces 
	leftTr = Ltrim$(temp)
	rightTr = Rtrim$(leftTr)

' The error message is captured also, so this is the indicator for the last record. The addresses
' appear in the text file along with the preceding MessagePrompt so the first 3 letters are truncated.
	While (Left$(rightTr , 3) <> ErrorMessage) 
		l = len(rightTr)	
		l = l - 4
		if l < 1 Then Goto OutOfArray		' check if there's something wrong
		table(i) = Right$(rightTr , l) 
		i = i + 1
		if i > 300  then goto OutOfArray

' Skip any blank lines
		FileReadNext temp, 0
		While (temp = "")
			FileReadNext temp, 0
		Wend

' Trunctate any leading or trailing spaces 
		leftTr = Ltrim$(temp)
		rightTr = Rtrim$(leftTr)
	Wend
OutOfArray:
End Sub

' ***********************************************************************************
' * Subroutine : SendMail
' * Purpose:  Send mail to a selected member from the list  
' *
' ***********************************************************************************

Sub  SendMail
	Dim subject As String
	Dim textFile As String

' Mail dialog
Again:

' msg is set to have the value of the mail address that was highlighted in the table.
msg = "Send mail to " & table(Dlg.lstbox)

' Dialog definition
DlgStart
	DlgTitle msg 
	DlgItemAsk  "Mail Subject :                                                                   ", subject, 1, 20
	DlgItemAsk  "Text file to send (leave empty to open notepad):", textFile, 2, 20
	DlgInquire 	
	DlgGetItemValue   1, subject    
	DlgGetItemValue   2, textfile
DlgEnd  	

' If textFile field  was left empty open Notepad to edit the mail
If textFile = "" Then 
		SysWinExec "notepad.exe"
		MsgBox "Click OK when you finished editing your document"
		goto again
End if

' Prepare mail for sending
MailPrepare "To", table(Dlg.lstbox), subject, textFile
MailSend

' Any problems ?
If (NM_ResultCode > 0) Then print NM_ResultText
End Sub